/**
 * @ignore
 * Binding token for mandatory value
 */
export type RequiredToken<T> = {
    symbol: symbol;
    type?: T;
    isOptional?: false;
};
/**
 * @ignore
 * Binding token for optional value
 */
export type OptionalToken<T> = {
    symbol: symbol;
    type?: T;
    isOptional: true;
    optionalValue: T;
};
/**
 * Binding token
 */
export type Token<T> = RequiredToken<T> | OptionalToken<T>;
/**
 * Token options
 */
export type TokenOptions = {
    /**
     * Key for token's symbol. It allows to create shareable tokens.
     */
    key: string;
    /** @ignore */
    description?: undefined;
} | {
    /** Description for better error messages */
    description?: string;
    /** @ignore */
    key?: undefined;
};
/**
 * Creates a new binding token.
 * @param description - Token description for better error messages.
 */
export declare function token<T>(description?: string): Token<T>;
/**
 * Creates a new binding token.
 * @param options - Token description for better error messages.
 */ export declare function token<T>(options?: TokenOptions): Token<T>;
/**
 * Decorate a token with an optional value.
 * This value is be used as default value in case a container does not have registered token.
 * @param token - Existed token.
 * @param optionalValue - Default value for the resolver.
 */
export declare function optional<T>(token: Token<T>, optionalValue: T): OptionalToken<T>;
export declare function optional<T>(token: Token<T>): OptionalToken<T | undefined>;
